//
//  lcd_draw.h	Part of Graphic LCD module I/O library, Primitive figure display interface
//  Copyright(C)2013-2023 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Configuration:
//        LCD_SCREEN_WIDTH		default=240
//        LCD_SCREEN_HEIGHT		default=320
//        LCD_UPSIDE_DOWN_MODE	default is disable
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.01.22 v0.0  First cording
//    2013.10.22 v0.1  Add color name
//    2014.01.09 v0.1  Changed a color code
//    2014.02.07 v0.1  Supported a LCD_LANDSCAPE_MODE
//    2014.03.15 v0.2  Rename source file: s95160_2.h -> lcd_draw.h + lcd_color.h
//    2014.03.16 v0.2  Add lcd_drawChar2(),lcd_drawStr2() and lcd_drawArc()
//    2018.01.19 v0.3  Supported a LCD_UPSIDE_DOWN_MODE
//    2022.02.15 v0.4  Change the way settings are made, and update history reset
//    2023.02.25 v0.5  Supported a ESP32 IDF
//

#ifndef LCD_DRAW_H_
#define LCD_DRAW_H_

#include "lcd_color.h"
#include "lcd_font.h"

//
//	Configuration
//

//						ADD START 2023.02.25
#ifdef ESP_PLATFORM				// ESP32 config symbol transform
    #include "sdkconfig.h"
    #ifdef CONFIG_LCD_MODULE_CP2401T
        #define   LCD_MODULE_CP2401T
    #endif
    #ifdef CONFIG_LCD_MODULE_YHY024006A
        #define   LCD_MODULE_YHY024006A
    #endif
    #ifdef CONFIG_LCD_MODULE_S95417AAA
        #define   LCD_MODULE_S95417AAA
    #endif
    #ifdef CONFIG_LCD_MODULE_VLCM240YP04
        #define   LCD_MODULE_VLCM240YP04
    #endif
    #ifdef CONFIG_LCD_MODULE_QJ240083S0
        #define   LCD_MODULE_QJ240083S0
    #endif
    #ifdef CONFIG_LCD_MODULE_S95160
        #define   LCD_MODULE_S95160
    #endif
    #ifdef CONFIG_LCD_MODULE_ATM0177
        #define   LCD_MODULE_ATM0177
    #endif
    #ifdef CONFIG_LCD_MODULE_ATM0177B3A
        #define   LCD_MODULE_ATM0177B3A
    #endif
    #ifdef CONFIG_LCD_MODULE_M014C9163
        #define   LCD_MODULE_M014C9163
    #endif
    #ifdef CONFIG_LCD_MODULE_M014C9163SPI
        #define   LCD_MODULE_M014C9163SPI
    #endif
    #ifdef CONFIG_LCD_MODULE_ZY_FGD1442701V1
        #define   LCD_MODULE_ZY_FGD1442701V1
    #endif
    #ifdef CONFIG_LCD_MODULE_Z144SN005
        #define   LCD_MODULE_Z144SN005
    #endif
    #ifdef CONFIG_LCD_MODULE_Z180SN009
        #define   LCD_MODULE_Z180SN009
    #endif
    #ifdef CONFIG_LCD_MODULE_Z180SN1720
        #define   LCD_MODULE_Z180SN1720
    #endif
    #ifdef CONFIG_LCD_MODULE_H161T01
        #define   LCD_MODULE_H161T01
    #endif
    #ifdef CONFIG_LCD_MODULE_L1F10085
        #define   LCD_MODULE_L1F10085
    #endif
    #ifdef CONFIG_LCD_MODULE_MSP2807
        #define   LCD_MODULE_MSP2807
    #endif
    #ifdef CONFIG_LCD_MODULE_ATM0130B3
        #define   LCD_MODULE_ATM0130B3
    #endif

    #ifdef CONFIG_LCD_LANDSCAPE_MODE
        #define   LCD_LANDSCAPE_MODE
    #endif
    #ifdef CONFIG_LCD_UPSIDE_DOWN_MODE
        #define   LCD_UPSIDE_DOWN_MODE
    #endif
#endif
//						ADD END 2023.02.25

#if defined(LCD_MODULE_CP2401T) || defined(LCD_MODUL_YHY024006A) || defined(LCD_MODULE_S95417AAA) || defined(LCD_MODULE_VLCM240YP04) || defined(LCD_MODULE_QJ240083S0)
  #ifndef LCD_LANDSCAPE_MODE
    #define LCD_SCREEN_WIDTH	240
    #define LCD_SCREEN_HEIGHT	320
  #else
    #define LCD_SCREEN_WIDTH	320
    #define LCD_SCREEN_HEIGHT	240
  #endif
  #define LCD_DRIVER_ILI9325

#elif defined(LCD_MODULE_S95160)
  #ifndef LCD_LANDSCAPE_MODE
    #define LCD_SCREEN_WIDTH	240
    #define LCD_SCREEN_HEIGHT	320
  #else
    #define LCD_SCREEN_WIDTH	320
    #define LCD_SCREEN_HEIGHT	240
  #endif
  #define LCD_DRIVER_SSD1289

#elif defined(LCD_MODULE_ATM0177) || defined(LCD_MODULE_ATM0177B3A)
  #ifndef LCD_LANDSCAPE_MODE
    #define LCD_SCREEN_WIDTH	128
    #define LCD_SCREEN_HEIGHT	160
  #else
    #define LCD_SCREEN_WIDTH	160
    #define LCD_SCREEN_HEIGHT	128
  #endif
  #define LCD_DRIVER_ILI9163V

#elif defined(LCD_MODULE_M014C9163) || defined(LCD_MODULE_M014C9163SPI)
  #define LCD_SCREEN_WIDTH		128
  #define LCD_SCREEN_HEIGHT		128
  #define LCD_DRIVER_ILI9163B

#elif defined(LCD_MODULE_Z144SN005) || defined(LCD_MODULE_ZY_FGD1442701V1)
  #define LCD_SCREEN_WIDTH		128
  #define LCD_SCREEN_HEIGHT		128
  #define LCD_DRIVER_ST7735S

#elif defined(LCD_MODULE_Z180SN009) || defined(LCD_MODULE_Z180SN1720)
  #ifndef LCD_LANDSCAPE_MODE
  #define LCD_SCREEN_WIDTH		128
  #define LCD_SCREEN_HEIGHT		160
  #else
  #define LCD_SCREEN_WIDTH		160
  #define LCD_SCREEN_HEIGHT		128
  #endif
  #define LCD_DRIVER_ST7735S
//#define LCD_DRIVER_ILI9163V

#elif defined(LCD_MODULE_H161T01)
  #define LCD_SCREEN_WIDTH		130
  #define LCD_SCREEN_HEIGHT		130
  #define LCD_DRIVER_SSD1283A

#elif defined(LCD_MODULE_L1F10085)
  #define LCD_SCREEN_WIDTH		128
  #define LCD_SCREEN_HEIGHT		128
  #define LCD_DRIVER_S1D15G10

#elif defined(LCD_MODULE_MSP2807)
  #define LCD_SCREEN_WIDTH		240
  #define LCD_SCREEN_HEIGHT		320
  #define LCD_DRIVER_ILI9341

#elif defined(LCD_MODULE_ATM0130B3)
  #define LCD_SCREEN_WIDTH		240
  #define LCD_SCREEN_HEIGHT		240
  #define LCD_DRIVER_ST7789V2

#else
  #error "Not supported LCD module"
#endif

//
//	API
//
extern void	lcd_drawOpen(void);
extern void lcd_drawRect(short x, short y, short width, short height, unsigned short color);
extern void lcd_drawLine(short x1, short y1, short x2, short y2, short line_size, unsigned short color);
extern void lcd_drawImage(short x, short y, short width, short height, unsigned short *image);
extern void lcd_drawChar(short x, short y, unsigned short face_color, unsigned short back_color, char c);
extern void lcd_drawStr(short x, short y, unsigned short face_color, unsigned short back_color, char *str);
extern void lcd_drawInit(void);
extern void lcd_drawChar2(short x, short y, unsigned short face_color, unsigned short back_color, char c, const struct lcd_font *font);		// ADD 2014.03.16
extern void lcd_drawStr2(short x, short y, unsigned short face_color, unsigned short back_color, char *str, const struct lcd_font *font);	// ADD 2014.03.16
extern void lcd_drawArc(short x, short y, short r, float startRad, float endRad, short line_size, unsigned short color);					// ADD 2014.03.16


#endif /* LCD_DRAW_H_ */
