//
//  lcd_draw_port.h	Part of Graphic LCD module I/O library, Transport layer
//  Physical device interface
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.19 v0.0  First cording
//

#ifndef LCD_DRAW_PORT_1_H_
#define LCD_DRAW_PORT_1_H_

#include <stddef.h>

//
// Prototype
//
extern void				lcd_portStart(void);
extern void				lcd_portStop(void);
extern void				lcd_portWait(int ms);

//
// 16bit I/O
//
extern void 			lcd_portWrite(int rs, unsigned short db);
//extern unsigned short	lcd_portRead(int rs);

#define					lcd_portWriteCmd(cmd)				lcd_portWrite(0, cmd)
#define					lcd_portWriteData(data)				lcd_portWrite(1, data)
#define					lcd_portWriteCmdData(cmd, data)		lcd_portWriteCmd(cmd); lcd_portWriteData(data)
//#define				lcd_portReadStatus()				lcd_portRead(0)
//#define				lcd_portReadData()					lcd_portRead(1)

//
// 8bit I/O
//
extern void 			lcd_portWrites8(int rs, size_t len, unsigned char *db);
#define 				lcd_portWrite8(rs, db)					{unsigned char c; c=(db); lcd_portWrites8(rs, 1, &c); }
//extern unsigned char	lcd_portRead8(int rs);
//extern int			lcd_portReads8(int rs, size_t len, unsigned char *db);

#define					lcd_portWriteCmd8(cmd)					lcd_portWrite8(0, cmd)
#define					lcd_portWriteData8(data)				lcd_portWrite8(1, data)
#define					lcd_portWriteCmdData8(cmd, data)		lcd_portWriteCmd8(cmd); lcd_portWriteData8(data)
#define					lcd_portWriteDatas8(len, data)			lcd_portWrites8(1, len, data)
#define					lcd_portWriteCmdDatas8(cmd, len, data)	lcd_portWriteCmd8(cmd); lcd_portWriteDatas8(len, data)
//#define				lcd_portReadStatus8()					lcd_portRead8(0)
//#define				lcd_portReadData8()						lcd_portRead8(1)

#endif	// LCD_DRAW_PORT_1_H_
