//
//  lcd_spec_ILI9163V.h	ILITEK a-Si TFT LCD Single Chip Driver ILI9163 command code
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//   Reference:
//      ILI TECHNOLOGY CORP.
//      a-Si TFT LCD Single Chip Driver 132RGBx162 Resolution and 262K color
//          Datasheet Specification
//              Version: V100
//              Document No.: ILI9163V_DTS_V100_20130416
//              Date: 2013.04.16
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.05 v0.0  First cording
//

#ifndef LCD_SPEC_ILI9163V_H_
#define LCD_SPEC_ILI9163V_H_

//
// Command code (index value)
//
#define LCD_COMMAND_NOP								0x00
#define LCD_COMMAND_SoftwareReset					0x01
#define LCD_COMMAND_ReadDisplayIdentificationInformation	0x04	// 4byte
#define LCD_COMMAND_ReadDisplayStatus				0x09	// 5byte
#define LCD_COMMAND_ReadDisplayPowerMode			0x0A	// 2byte
#define LCD_COMMAND_ReadDisplayMADCTL				0x0B	// 2byte
#define LCD_COMMAND_ReadDisplayPixelFormat			0x0C	// 2byte
#define LCD_COMMAND_ReadDisplayImageMode			0x0D	// 2byte
#define LCD_COMMAND_ReadDisplaySignalMode			0x0E	// 2byte
#define LCD_COMMAND_ReadDisplaySignalMode2			0x0F	// 2byte
#define LCD_COMMAND_SleepIn							0x10
#define LCD_COMMAND_SleepOut						0x11
#define LCD_COMMAND_PartialModeOn					0x12
#define LCD_COMMAND_NormalDisplayModeOn				0x13
#define LCD_COMMAND_DisplayInversionOff				0x20
#define LCD_COMMAND_DisplayInversionOn				0x21
#define LCD_COMMAND_GammaSet						0x26	// 1byte
#define LCD_COMMAND_DisplayOff						0x28
#define LCD_COMMAND_DisplayOn						0x29
#define LCD_COMMAND_ColumnAddressSet				0x2A	// 4byte XS,XE
#define LCD_COMMAND_PageAddressSet					0x2B	// 4byte YS,YE
#define LCD_COMMAND_MemoryWrite						0x2C	// *Byte
#define LCD_COMMAND_ColorSetting					0x2D	// 128byte
#define LCD_COMMAND_MemoryRead						0x2E	// *byte
#define LCD_COMMAND_PartialArea						0x30	// 4byte PSL,PEL
#define LCD_COMMAND_VerticalScrollingDefinition		0x33	// 6byte TFA,VSA,BFA
#define LCD_COMMAND_TearingEffectLineOff			0x34
#define LCD_COMMAND_TearingEffectLineOn				0x35	// 1byte
#define LCD_COMMAND_MemoryAccessControl				0x36	// 1byte
#define LCD_COMMAND_VerticalScrollingStartAddress	0x37	// 2byte
#define LCD_COMMAND_IdleModeOff						0x38
#define LCD_COMMAND_IdleModeOn						0x39
#define LCD_COMMAND_InterfacePixelFormat			0x3A	// 1byte
#define LCD_COMMAND_FrameRateControl1				0xB1	// 2byte
#define LCD_COMMAND_FrameRateControl2				0xB2	// 2byte
#define LCD_COMMAND_FrameRateControl3				0xB3	// 2byte
#define LCD_COMMAND_DisplayInversionControl			0xB4	// 1byte
#define LCD_COMMAND_RGBInterfaceBlankingPorchSetting	0xB5	// 3byte
#define LCD_COMMAND_DisplayFuctionset5				0xB6	// 2byte
#define LCD_COMMAND_SourceDriverDirectionControl	0xB7	// 1byte
#define LCD_COMMAND_GateDriverDirectionControl		0xB8	// 1byte
#define LCD_COMMAND_PowerControl1					0xC0	// 2byte
#define LCD_COMMAND_PowerControl2					0xC1	// 1byte
#define LCD_COMMAND_PowerControl3					0xC2	// 1byte
#define LCD_COMMAND_PowerControl4					0xC3	// 1byte
#define LCD_COMMAND_PowerControl5					0xC4	// 1byte
#define LCD_COMMAND_VCOMControl1					0xC5	// 2byte
#define LCD_COMMAND_VCOMOffsetControl				0xC7	// 1byte
#define LCD_COMMAND_WriteID4Value					0xD3	// 5byte
#define LCD_COMMAND_NVMemoryFunctionController1		0xD5	// 2byte
#define LCD_COMMAND_NVMemoryFunctionController2		0xD6	// 2byte
#define LCD_COMMAND_NVMemoryFunctionController3		0xD7	// 2byte
#define LCD_COMMAND_ReadID1							0xDA	// 2byte
#define LCD_COMMAND_ReadID2							0xDB	// 2byte
#define LCD_COMMAND_ReadID3							0xDC	// 2byte
#define LCD_COMMAND_PositiveGammaCorrectionSetting	0xE0	// 15byte
#define LCD_COMMAND_NegativeGammaCorrectionSetting	0xE1	// 15byte
#define LCD_COMMAND_GAM_R_SEL						0xF2	// 1byte

//
// Parameter
//
// Memory Access Control (36h)
#define LCD_DATA_MY		(1 << 7)	// Row Address Order
#define LCD_DATA_MX		(1 << 6)	// Column Address Order
#define LCD_DATA_MV		(1 << 5)	// Page/Column Selection
#define LCD_DATA_ML		(1 << 4)	// Vertical Order
#define LCD_DATA_RGB	(0)			// RGB color filter panel
#define LCD_DATA_BGR	(1 << 3)	// BGR color filter panel
#define LCD_DATA_MH		(1 << 2)	// Display data latch order

// Interface Pixel Format (3Ah)
#define LCD_DATA_VIPF(n)			(n << 4)	// RGB Interface Color Format
#define LCD_DATA_VIPF_16bitPixel	LCD_DATA_VIPF(0b0101)
#define LCD_DATA_VIPF_18bitPixel	LCD_DATA_VIPF(0b0110)
#define LCD_DATA_VIPF_18bitPixel_3times	LCD_DATA_VIPF(0b1110)
#define LCD_DATA_IFPF(n)			(n << 0)	// Control Interface Color Format
#define LCD_DATA_IFPF_12bitPixel	LCD_DATA_IFPF(0b011)
#define LCD_DATA_IFPF_16bitPixel	LCD_DATA_IFPF(0b101)
#define LCD_DATA_IFPF_18bitPixel	LCD_DATA_IFPF(0b110)


#endif /* LCD_SPEC_ILI9163V_H_ */
