//
//  lcd_spec_ILI9325.h	ILITEK a-Si TFT LCD Single Chip Driver ILI9325 command code
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//   Refernce:
//      ILI TECHNOLOGY CORP.
//      a-Si TFT LCD Single Chip Driver 240RGBx320 Resolution and 262K color
//          Datasheet Specification
//              Version: V0.43
//              Document No.: ILI9325DS_V0.43.pdf
//              Date: 2008.9.10
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.18 v0.0  First cording
//

#ifndef LCD_SPEC_ILI9325_H_
#define LCD_SPEC_ILI9325_H_

//
// Command code (index value)
//
#define LCD_COMMAND_StartOscillation					0x00
#define LCD_COMMAND_DriverCodeRead						0x00
#define LCD_COMMAND_DriverOutputControl1				0x01
#define LCD_COMMAND_LCDDrivingControl					0x02
#define LCD_COMMAND_EntryMode							0x03
#define LCD_COMMAND_ResizeControl						0x04
#define LCD_COMMAND_DisplayControl1						0x07
#define LCD_COMMAND_DisplayControl2						0x08
#define LCD_COMMAND_DisplayControl3						0x09
#define LCD_COMMAND_DisplayControl4						0x0a
#define LCD_COMMAND_RGBDisplayInterfaceControl1			0x0c
#define LCD_COMMAND_FrameMakerPosition					0x0d
#define LCD_COMMAND_RGBDisplayInterfaceControl2			0x0f
#define LCD_COMMAND_PowerControl1						0x10
#define LCD_COMMAND_PowerControl2						0x11
#define LCD_COMMAND_PowerControl3						0x12
#define LCD_COMMAND_PowerControl4						0x13
#define LCD_COMMAND_HorizentalGRAMAddressSet			0x20
#define LCD_COMMAND_VerticalGRAMAddressSet				0x21
#define LCD_COMMAND_WriteDataToGRAM						0x22
#define LCD_COMMAND_PowerControl7						0x29
#define LCD_COMMAND_FrameRateAndColorControl			0x2b
#define LCD_COMMAND_GammaControl1						0x30
#define LCD_COMMAND_GammaControl2						0x31
#define LCD_COMMAND_GammaControl3						0x32
#define LCD_COMMAND_GammaControl4						0x35
#define LCD_COMMAND_GammaControl5						0x36
#define LCD_COMMAND_GammaControl6						0x37
#define LCD_COMMAND_GammaControl7						0x38
#define LCD_COMMAND_GammaControl8						0x39
#define LCD_COMMAND_GammaControl9						0x3c
#define LCD_COMMAND_HorizentalAddressStartPossition		0x50
#define LCD_COMMAND_HorizentalAddressEndPossition		0x51
#define LCD_COMMAND_VerticalAddressStartPossition		0x52
#define LCD_COMMAND_VerticalAddressEndPossition			0x53
#define LCD_COMMAND_DriverOutputControl2				0x60
#define LCD_COMMAND_BaseImageDisplayControl				0x61
#define LCD_COMMAND_VerticalScrollControl				0x6a
#define LCD_COMMAND_PartialImage1DisplayPosition		0x80
#define LCD_COMMAND_PartialImage1AreaStartLine			0x81
#define LCD_COMMAND_PartialImage1AreaEndLine			0x82
#define LCD_COMMAND_PartialImage2DisplayPosition		0x83
#define LCD_COMMAND_PartialImage2AreaStartLine			0x84
#define LCD_COMMAND_PartialImage2AreaEndLine			0x85
#define LCD_COMMAND_PanelInterfaceControl1				0x90
#define LCD_COMMAND_PanelInterfaceControl2				0x92
#define LCD_COMMAND_PanelInterfaceControl4				0x95
#define LCD_COMMAND_OTPVCMProgrammingControl			0xa1
#define LCD_COMMAND_OTPVCMStatusAndEnable				0xa2
#define LCD_COMMAND_OTPProgrammingIDKey					0xa5

//
// Parameter
//
// Driver Output Control (R01h)
#define LCD_DATA_SS				0b0000000100000000	// Select the shift direction of outputs from the source driver.
#define LCD_DATA_SM				0b0000010000000000	// Sets the gate driver pin arrangement in combination with the GS bit (R60h) to select the optimal scan mode for the module.

// LCD Driving Wave Control (R02h)
#define LCD_DATA_BC				0b0000001000000000	// Frame/Field inversion
#define LCD_DATA_EOR			0b0000000100000000	// EOR = 1 and B/C=1 to set the line inversion.

// Entry Mode (R03h)
#define LCD_DATA_AM				0b0000000000001000	// Control the GRAM update direction.
#define LCD_DATA_AM_HORIZONTAL	0b0000000000000000
#define LCD_DATA_AM_VERTICAL	0b0000000000001000
#define LCD_DATA_ID(n)			(n << 4)			// Control the address counter (AC) to automatically increase or decrease by 1 when update one pixel display data. Refer to the following figure for the details.
#define LCD_DATA_ORG			0b0000000010000000	// Moves the origin address according to the ID setting when a window address area is made.
//#define LCD_DATA_HWM			0b0000001000000000	// High speed GRAM access.
#define LCD_DATA_BGR			0b0001000000000000	// Swap the R and B order of written data.
#define LCD_DATA_DFM			0b0100000000000000	// Set the mode of transferring data to the internal RAM when TRI = “1”. See the following figures for details.
#define LCD_DATA_TRI			0b1000000000000000	// data are transferred to the internal RAM in 8-bit x 3 transfers mode via the 8-bit interface.

// Resizing Control Register (R04h)

// Display Control 1 (R07h)
#define LCD_DATA_D(n)			(n << 0)			//
#define LCD_DATA_CL				0b0000000000001000	// the 8-color display mode is selected.
#define LCD_DATA_DTE			0b0000000000010000	// Set the output level of gate driver G1 ~ G320 as follows
#define LCD_DATA_GON			0b0000000000100000	// Set the output level of gate driver G1 ~ G320 as follows
#define LCD_DATA_BASEE			0b0000000100000000	// Base image display enable bit. When BASEE = “0”, no base image is displayed.
#define LCD_DATA_PTDE(n)		(n << 12)			// Partial image 2 and Partial image 1 enable bits

// Display Control 2 (R08h)
#define LCD_DATA_FP(n)			(n << 8)			// Number of lines for Front Porch
#define LCD_DATA_BP(n)			(n << 0)			// Number of lines for Back Porch

// Display Control 3 (R09h)
#define LCD_DATA_ISC(n)			(n << 0)			// Specify the scan cycle interval of gate driver in non-display area when PTG[1:0]=”10” to select interval scan.
#define LCD_DATA_PTG(n)			(n << 4)			// Set the scan mode in non-display area.
#define LCD_DATA_PTS(n)			(n << 8)			// Set the source output level in non-display area drive period (front/back porch period and blank area between partial displays).

// Display Control 4 (R0Ah)
#define LCD_DATA_FM(n)			(n << 0)			// Set the output interval of FMARK signal according to the display data rewrite cycle and data transfer rate.
#define LCD_DATA_FMARKOE		0b0000000000001000	// ILI9325 starts to output FMARK signal in the output interval set by FMI[2:0] bits.

// RGB Display Interface Control 1 (R0Ch)

// Frame Marker Position (R0Dh)

// RGB Display Interface Control 2 (R0Fh)

// Power Control 1 (R10h)
#define LCD_DATA_STB			0b0000000000000001	// ILI9325 enters the standby mode and the display operation stops except the GRAM power supply to reduce the power consumption.
#define LCD_DATA_SLP			0b0000000000000010	// ILI9325 enters the sleep mode and the display operation stops except the RC oscillator to reduce the power consumption.
//#define LCD_DATA_DSTB			0b0000000000000100	// ILI9325 enters the deep standby mode. In deep standby mode, the internal logic power supply is turned off to reduce power consumption.
#define LCD_DATA_AP(n)			(n << 4)			// Adjusts the constant current in the operational amplifier circuit in the LCD power supply circuit.
													//    AP                        0    1    2    3    4    5    6    7
													//    Gamma driver amplifiers  Halt 1.00 1.00 1.00 0.75 0.75 0.75 0.50
													//    Source driver amplifiers Halt 1.00 0.75 0.50 1.00 0.75 0.50 0.50
#define LCD_DATA_APE			0b0000000010000000	// Power supply enable bit.
#define LCD_DATA_BT(n)			(n << 8)			// Sets the factor used in the step-up circuits.
#define LCD_DATA_SAP			0b0001000000000000	// Source Driver output control

// Power Control 2 (R11h)
#define LCD_DATA_VC(n)			(n << 0)			// Sets the ratio factor of Vci to generate the reference voltages Vci1.
													//    Vci1 = Vci * ((VC > 6) ? (1.0) : (0.95 - VC * 0.05))
#define LCD_DATA_DC0(n)			(n << 4)			// Selects the operating frequency of the step-up circuit 1.
													//    Frequency = Fosc / (2 ^ DC0)
#define LCD_DATA_DC1(n)			(n << 8)			// Selects the operating frequency of the step-up circuit 2.
													//    Frequency = Fosc / (2 ^ (DC1 + 2))

// Power Control 3 (R12h)
#define LCD_DATA_VRH(n)			(n << 0)			// Set the amplifying rate (1.6 ~ 1.9) of Vci applied to output the VREG1OUT level
													//    VCIRE==0 -> VREG1OUT = Vci * VRH * 0.05 + ((VRH < 8) ? 1.95 : 1.2)
													//    VCIRE==1 -> VREG1OUT = VRH * 0.125 + ((VRH < 8) ? 4.875 : 3.0)
#define LCD_DATA_PON			0b0000000000010000	// Control ON/OFF of circuit3 (VGL) output.
#define LCD_DATA_VCIRE			0b0000000010000000	// Select the external reference voltage Vci or internal reference voltage VCIR.

// Power Control 4 (R13h)
#define LCD_DATA_VDV(n)			(n << 8)			// Select the factor of VREG1OUT to set the amplitude of Vcom alternating voltage from 0.70 to 1.24 x VREG1OUT.
													//    Vcom amplitude = VREG1OUT * (( VDV < 16 ) ? (VDV * 0.02 + 0.7) : (VDV * 0.02 + 0.62))

// GRAM Horizontal/Vertical Address Set (R20h, R21h)
#define LCD_DATA_AD_H(n)		(n)					// Horizontal
#define LCD_DATA_AD_V(n)		(n)					// Vertical

// Power Control 7 (R29h)
#define LCD_DATA_VCM(n)			(n << 0)			// Set the internal VcomH voltage.
													//    VCOMH = VREG1OUT * (VCM * 0.005 + 0.685)

// Frame Rate and Color Control (R2Bh)
#define LCD_DATA_FRS(n)			(n << 0)			// Set the frame rate when the internal resistor is used for oscillator circuit.

// Horizontal and Vertical RAM Address Position (R50h, R51h, R52h, R53h)
#define LCD_DATA_HSA(n)			(n)					// Horizontal start
#define LCD_DATA_HEA(n)			(n)					// Horizontal end
#define LCD_DATA_VSA(n)			(n)					// Vertical start
#define LCD_DATA_VEA(n)			(n)					// Vertical end

// Gate Scan Control (R60h, R61h, R6Ah)
#define LCD_DATA_SCN(n)			(n << 0)			// The ILI9325 allows to specify the gate line from which the gate driver starts to scan by setting the SCN[5:0] bits.
#define LCD_DATA_NL(line)		((line/8-1) << 8)	// Sets the number of lines to drive the LCD at an interval of 8 lines.
#define LCD_DATA_NDL			0b0000000000000100	// Sets the source driver output level in the non-display area.
#define LCD_DATA_GS				0b1000000000000000	// Sets the direction of scan by the gate driver in the range determined by SCN[4:0] and NL[4:0].
#define LCD_DATA_REV			0b0000000000000001	// Enables the grayscale inversion of the imageby setting REV=1.
#define LCD_DATA_VLE			0b0000000000000010	// Vertical scroll display enable bit.
#define LCD_DATA_VL(n)			(n << 0)			// Sets the scrolling amount of base image.

// Panel Interface Control 1 (R90h)
#define LCD_DATA_RTN(n)			(n << 0)			// Sets 1H (line) clock number of internal clock operating mode. In this mode, ILI9325 display
#define LCD_DATA_DIV(n)			(n << 8)			// Sets the division ratio of internal clock frequency.

// Panel Interface Control 2 (R92h)
#define LCD_DATA_NOW(n)			(n << 0)			// Sets the gate output non-overlap period when ILI9325 display operation is synchronized with internal clock signal.

// Panel Interface Control 4 (R95h)
#define LCD_DATA_RTNE(n)		(n << 0)			// Sets 1H (line) clock number of RGB interface mode. In this mode, ILI9325 display operation is synchronized with RGB interface signals.
#define LCD_DATA_DIVE(n)		(n << 8)			// Sets the division ratio of DOTCLK when ILI9325 display operation is synchronized with RGB interface signals.

#endif	// LCD_SPEC_ILI9325_H_
