//
//  lcd_spec_ILI9341.h	ILITEK a-Si TFT LCD Single Chip Driver ILI9341 command code
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//   Reference:
//      ILI TECHNOLOGY CORP.
//      a-Si TFT LCD Single Chip Driver 240RGBx320 Resolution and 262K color
//          Specification
//              Version     : V1.09
//              Document No.: ILI9341_DS_V1.09.pdf
//              Date        : 2011.03.15
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.09 v0.0  First cording
//

#ifndef LCD_SPEC_ILI9341_H_
#define LCD_SPEC_ILI9341_H_

//
// Level 1 Command
//
#define LCD_COMMAND_NOP								0x00
#define LCD_COMMAND_SoftwareReset					0x01
#define LCD_COMMAND_ReadDisplayIdentificationInformation	0x04	// 4byte
#define LCD_COMMAND_ReadDisplayStatus				0x09	// 5byte
#define LCD_COMMAND_ReadDisplayPowerMode			0x0A	// 2byte
#define LCD_COMMAND_ReadDisplayMADCTL				0x0B	// 2byte
#define LCD_COMMAND_ReadDisplayPixelFormat			0x0C	// 2byte
#define LCD_COMMAND_ReadDisplayImageFormat			0x0D	// 2byte
#define LCD_COMMAND_ReadDisplaySignalMode			0x0E	// 2byte
#define LCD_COMMAND_ReadDisplaySelfDiagnosticResult			0x0F	// 2byte
#define LCD_COMMAND_EnterSleepMode					0x10
#define LCD_COMMAND_SleepOut						0x11
#define LCD_COMMAND_PartialModeON					0x12
#define LCD_COMMAND_NormalDisplayModeON				0x13
#define LCD_COMMAND_DisplayInversionOFF				0x20
#define LCD_COMMAND_DisplayInversionON				0x21
#define LCD_COMMAND_GammaSet						0x26	// 1byte
#define LCD_COMMAND_DisplayOFF						0x28
#define LCD_COMMAND_DisplayON						0x29
#define LCD_COMMAND_ColumnAddressSet				0x2A	// 4byte XS,XE
#define LCD_COMMAND_PageAddressSet					0x2B	// 4byte YS,YE
#define LCD_COMMAND_MemoryWrite						0x2C	// n Word
#define LCD_COMMAND_ColorSet						0x2D	// 128byte
#define LCD_COMMAND_MemoryRead						0x2E	// n Word
#define LCD_COMMAND_PartialArea						0x30	// 4byte PSL,PEL
#define LCD_COMMAND_VerticalScrollingDefinition		0x33	// 6byte TFA,VSA,BFA
#define LCD_COMMAND_TearingEffectLineOFF			0x34
#define LCD_COMMAND_TearingEffectLineON				0x35	// 1byte
#define LCD_COMMAND_MemoryAccessControl				0x36	// 1byte
#define LCD_COMMAND_VerticalScrollingStartAddress	0x37	// 2byte
#define LCD_COMMAND_IdleModeOFF						0x38
#define LCD_COMMAND_IdleModeON						0x39
#define LCD_COMMAND_PixelFormatSet					0x3A	// 1byte
#define LCD_COMMAND_WriteMemoryContinue				0x3C
#define LCD_COMMAND_ReadMemoryContinue				0x3E
#define LCD_COMMAND_SetTearScanline					0x44
#define LCD_COMMAND_GetScanline						0x45
#define LCD_COMMAND_WriteDisplayBrightness			0x51
#define LCD_COMMAND_ReadDisplayBrightness			0x52
#define LCD_COMMAND_WriteCTRLDisplay				0x53
#define LCD_COMMAND_ReadCTRLDisplay					0x54
#define LCD_COMMAND_WriteContentAdaptiveBrightnessControl	0x55
#define LCD_COMMAND_ReadContentAdaptiveBrightnessControl	0x56
#define LCD_COMMAND_WriteCABCMinimumBrightness		0x5E
#define LCD_COMMAND_ReadCABCMinimumBrightness		0x5F
#define LCD_COMMAND_ReadID1							0xDA	// 2byte
#define LCD_COMMAND_ReadID2							0xDB	// 2byte
#define LCD_COMMAND_ReadID3							0xDC	// 2byte

//
// Level 2 Command
//
#define LCD_COMMAND_RGBInterfaceSignalControl 		0xB0	//  byte
#define LCD_COMMAND_FrameRateControl1				0xB1	// 2byte
#define LCD_COMMAND_FrameRateControl2				0xB2	// 2byte
#define LCD_COMMAND_FrameRateControl3				0xB3	// 2byte
#define LCD_COMMAND_DisplayInversionControl			0xB4	// 1byte
#define LCD_COMMAND_BlankingPorchSetting			0xB5	// 3byte
#define LCD_COMMAND_DisplayFuctionControl			0xB6	// 4byte
#define LCD_COMMAND_EntryModeSet					0xB7	// 1byte
#define LCD_COMMAND_BacklightControl1				0xB8	// 1byte
#define LCD_COMMAND_BacklightControl2				0xB9	// 1byte
#define LCD_COMMAND_BacklightControl3				0xBA	// 1byte
#define LCD_COMMAND_BacklightControl4				0xBB	// 1byte
#define LCD_COMMAND_BacklightControl5				0xBC	// 1byte
#define LCD_COMMAND_BacklightControl6				0xBD	// 1byte
#define LCD_COMMAND_BacklightControl7				0xBE	// 1byte
#define LCD_COMMAND_BacklightControl8				0xBF	// 1byte
#define LCD_COMMAND_PowerControl1					0xC0	// 2byte
#define LCD_COMMAND_PowerControl2					0xC1	// 1byte
#define LCD_COMMAND_VCOMControl1					0xC5	// 2byte
#define LCD_COMMAND_VCOMControl2					0xC7	// 1byte
#define LCD_COMMAND_NVMemoryWrite					0xD0	// 2byte
#define LCD_COMMAND_NVMemoryProtectionKey			0xD1	// 2byte
#define LCD_COMMAND_NVMemoryStatusRead				0xD2	// 2byte
#define LCD_COMMAND_ReadID4							0xD3	// 2byte
#define LCD_COMMAND_PositiveGammaCorrection			0xE0	// 15byte
#define LCD_COMMAND_NegativeGammaCorrection			0xE1	// 15byte
#define LCD_COMMAND_DigitalGammaControl1			0xE2	// 15byte
#define LCD_COMMAND_DigitalGammaControl2			0xE3	// 15byte
#define LCD_COMMAND_InterfaceControl				0xF6	// 1byte

//
// Extended register Command
//
#define LCD_COMMAND_PowerControlA					0xCB	// 1byte
#define LCD_COMMAND_PowerControlB					0xCF	// 1byte
#define LCD_COMMAND_DriverTimingControlA			0xE8	// 1byte
#define LCD_COMMAND_DriverTimingControlB			0xEA	// 1byte
#define LCD_COMMAND_PowerOnSequenceControl			0xED	// 1byte
#define LCD_COMMAND_Enable3gamma					0xF2	// 1byte
#define LCD_COMMAND_PumpRatioControl				0xF7	// 1byte

//
// Parameter
//
// Gamma Set (26h)
#define LCD_DATA_GC0	0x01		// Gamma curve 1 (G2.2)

// Column Address Set (2Ah)
#define LCD_DATA1_SC(x)		(x >> 8)	// Start
#define LCD_DATA2_SC(x)		(x & 0xff)
#define LCD_DATA3_EC(x)		(x >> 8)	// End
#define LCD_DATA4_EC(x)		(x & 0xff)

// Page Address Set (2Bh)
#define LCD_DATA1_SP(y)		(y >> 8)	// Start
#define LCD_DATA2_SP(y)		(y & 0xff)
#define LCD_DATA3_EP(y)		(y >> 8)	// End
#define LCD_DATA4_EP(y)		(y & 0xff)

// Partial Area (30h)
#define LCD_DATA1_SR(y)		(y >> 8)	// Start
#define LCD_DATA2_SR(y)		(y & 0xff)
#define LCD_DATA3_ER(y)		(y >> 8)	// End
#define LCD_DATA4_ER(y)		(y & 0xff)

// Vertical Scrolling Definition (33h)
#define LCD_DATA1_TFA(y)	(y >> 8)	// Top Fixed Area
#define LCD_DATA2_TFA(y)	(y & 0xff)
#define LCD_DATA3_VSA(y)	(y >> 8)	// Vertical Scrolling Area
#define LCD_DATA4_VSA(y)	(y & 0xff)
#define LCD_DATA5_BFA(y)	(y >> 8)	// Bottom Fixed Area
#define LCD_DATA6_BFA(y)	(y & 0xff)

// Tearing Effect Line ON (35h)
#define LCD_DATA_M			0x01	// The Tearing Effect Output Line consists of both V-Blanking and H-Blanking information

// Memory Access Control (36h)
#define LCD_DATA_MY			(1 << 7)	// Row Address Order
#define LCD_DATA_MX			(1 << 6)	// Column Address Order
#define LCD_DATA_MV			(1 << 5)	// Page/Column Selection
#define LCD_DATA_ML			(1 << 4)	// Vertical Order
#define LCD_DATA_ML_Top_Bottom	(0<<4)	//
#define LCD_DATA_ML_Bottom_Top	(1<<4)	//
#define LCD_DATA_BGR		(1 << 3)	// BGR color filter panel
#define LCD_DATA_RGB		(0)			// RGB color filter panel
#define LCD_DATA_MH			(1 << 2)	// Display data latch order
#define LCD_DATA_MH_Left_Right	(0<<2)	//
#define LCD_DATA_MH_Right_Left	(1<<2)	//

// Vertical Scrolling Start Address (37h)
#define LCD_DATA1_VSP(y)	(y >> 8)
#define LCD_DATA2_VSP(y)	(y & 0xff)

// Pixel Format Set (3Ah)
#define LCD_DATA_DPI(n)		(n << 4)	// RGB Interface Color Format
#define LCD_DATA_DPI_16bit	LCD_DATA_DPI(0b101)
#define LCD_DATA_DPI_18bit	LCD_DATA_DPI(0b110)
#define LCD_DATA_DBI(n)		(n << 0)	// Control Interface Color Format
#define LCD_DATA_DBI_16bit	LCD_DATA_DBI(0b101)
#define LCD_DATA_DBI_18bit	LCD_DATA_DBI(0b110)

// Set_Tear_Scanline (44h)
#define LCD_DATA1_STS(n)	(n >> 8)	//
#define LCD_DATA2_STS(n)	(n & 0xff)

// Get_Scanline (45h)
#define LCD_DATA1_GTS(n)	(n >> 8)	//
#define LCD_DATA2_GTS(n)	(n & 0xff)

// Write Display Brightness (51h)
#define LCD_DATA_DBV(n)		(n << 0)	// 0..255

// Write CTRL Display (53h)
#define LCD_DATA_BCTRL		(1 << 5)	// Brightness Control Block On/Off
#define LCD_DATA_DD			(1 << 3)	// Display Dimming On/Off
#define LCD_DATA_BL			(1 << 2)	// Backlight Control On/Off

// Write Content Adaptive Brightness Control (55h)
#define LCD_DATA_C(n)		(n << 0)	// 0..3
#define LCD_DATA_C_Off					LCD_DATA_C(0)
#define LCD_DATA_C_UserInterfaceImage	LCD_DATA_C(1)
#define LCD_DATA_C_StillPicture			LCD_DATA_C(2)
#define LCD_DATA_C_MovingImage			LCD_DATA_C(3)

// Write CABC Minimum Brightness (5Eh)
#define LCD_DATA_CMB(n)		(n << 0)	// 0..255

// Display Function Control (B6h)
#define LCD_DATA1_PTG(n)	(n >> 2)	// Set the scan mode in non-display area.
#define LCD_DATA1_PTG_NormalScan			LCD_DATA1_PTG(0)	//
#define LCD_DATA1_PTG_SettingProhibited		LCD_DATA1_PTG(1)	//
#define LCD_DATA1_PTG_IntervalScan			LCD_DATA1_PTG(2)	// *
#define LCD_DATA1_PT(n)		(n >> 0)	// Determine source/VCOM output in a non-display area in the partial display mode.
#define LCD_DATA1_PT_V63_V0_VCOML_VCOMH		LCD_DATA1_PT(0)		//
#define LCD_DATA1_PT_V0_V63_VCOML_VCOMH		LCD_DATA1_PT(1)		//
#define LCD_DATA1_PT_AGND_AGND_AGND_AGND	LCD_DATA1_PT(2)		// *
#define LCD_DATA1_PT_HiZ_HiZ_AGND_AGND		LCD_DATA1_PT(3)		//
#define LCD_DATA2_SS		(1 << 5)	// Select the shift direction of outputs from the source driver.
#define LCD_DATA2_SS_S1_S720				0					// *
#define LCD_DATA2_SS_S720_S1				LCD_DATA2_SS		//
#define LCD_DATA2_REV		(1 << 7)	// Select whether the liquid crystal type is normally white type or normally black type.
#define LCD_DATA2_REV_NormallyBlack			0					//
#define LCD_DATA2_REV_NormallyWhite			LCD_DATA2_REV		// *
#define LCD_DATA2_ISC(n)	(n << 0)	// [2] Specify the scan cycle interval of gate driver in non-display area when PTG [1:0] =”10” to select interval scan.
#define LCD_DATA2_GS		(1 << 6)	// [0] Sets the direction of scan by the gate driver in the range determined by SCN [4:0] and NL [4:0].
#define LCD_DATA2_SM		(1 << 4)	// [0] Sets the gate driver pin arrangement in combination with the GS bit to select the optimal scan mode for the module.
#define LCD_DATA3_NL(n)		(n << 0)	// [39] Sets the number of lines to drive the LCD at an interval of 8 lines.
#define LCD_DATA4_PCDIV(n)	(n << 0)	// [xx] Tosc=DOTCLK/(2*(PCDIV+1))


#endif /* LCD_SPEC_ILI9341_H_ */
