//
//  lcd_spec_SSD1283A.h	Solomon Systech LCD Driver SSD1289 command code
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//  Refernce:
//      SOLOMON SYSTECH
//      SSD1283A
//      132 RGB x 132 TFT Driver
//      Integrated Power, Gate, Source Driver with built-in RAM
//          Data sheet
//              Version     : Rev 0.12
//              Document No.: none
//              Date        : Jul 2005
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.06 v0.0  First cording
//

#ifndef LCD_SPEC_SSD1283A_H_
#define LCD_SPEC_SSD1283A_H_

//
// Command code (index value)
//
#define LCD_COMMAND_Oscillator						0x00
#define LCD_DATA_Oscillator							0b0000000000000000
#define LCD_DATA_OSCEN				(1<<0)		// The oscillator will be turned on

#define LCD_COMMAND_DriverOutputControl				0x01
#define LCD_DATA_DriverOutputControl				0b0000000000000000
#define LCD_DATA_REV				(1<<13)		// Displays all character and graphics display sections with reversal
#define LCD_DATA_CAD				(1<<12)		// Set up based on retention capacitor configuration of the TFT panel
#define LCD_DATA_BGR				(1<<11)		// <B><G><R> color is assigned (0=RGB)
#define LCD_DATA_SM					(1<<10)		// Change scanning order of gate driver
#define LCD_DATA_TB					(1<<9)		// Selects the output shift <G0 to G131> direction of the gate driver (0=G131 to G0)
#define LCD_DATA_RL					(1<<8)		// Selects the output shift <S0 to S395> direction of the source driver
#define LCD_DATA_MUX(n)				((n)<<0)	// Specify number of lines for the LCD driver (8bits)

#define LCD_COMMAND_LCDdrivingWaveformControl		0x02
#define LCD_DATA_DLCDdrivingWaveformControl			0b0000000000000000
#define LCD_DATA_FLD				(1<<11)		// Set display in interlace drive mode to protect from flicker. It splits one frame into 3 fields and drive.
#define LCD_DATA_BC					(1<<9)		// a N-line inversion waveform is generated and alternates in each N lines specified by bits EOR and NW.
#define LCD_DATA_EOR				(1<<8)		// the odd/even frame-select signals and the N-line inversion signals are EORed for alternating drive.
#define LCD_DATA_NW(n)				((n)<<0)	// Specify the number of lines that will alternate at the N-line inversion setting (B/C = 1) (7bits)

#define LCD_COMMAND_EntryMode						0x03
#define LCD_DATA_EntryMode							0b0000000000000000
#define LCD_DATA_VSMode				(1<<15)		// the frame frequency will be dependent on VSYNC (Dmode=00)
#define LCD_DATA_DFM(n)				((n)<<13)	// Set the color display mode
#define LCD_DATA_DFM_65k			(3<<13)		// [POR]
#define LCD_DATA_DFM_262k			(2<<13)		//
#define LCD_DATA_TRANS				(1<<12)		// transparent display is allowed.(Dmode=1x)
#define LCD_DATA_OEDef				(1<<11)		// OE defines the display window.
#define LCD_DATA_WMode				(1<<10)		// write ram from generic interface.(0=write ram from normal data bus)
#define LCD_DATA_DMode(n)			((n)<<8)	// SSD1283A allows data display from ram data or from generic input data
#define LCD_DATA_DMode_Ram			(0<<8)		// [POR]
#define LCD_DATA_DMode_GenericInput	(1<<8)		//
#define LCD_DATA_TY(n)				((n)<<6)	// In 262k color mode, 16 bit parallel interface, there are three types of methods in writing data into the ram, Type A, B and C are described as below.
#define LCD_DATA_TY_TypeA			(0<<6)		// 3word(RG,BR,GB), 2pixel
#define LCD_DATA_TY_TypeB			(1<<6)		// 2word(RG,xB)
#define LCD_DATA_TY_TypeC			(2<<6)		// 2word(RG,Bx)
#define LCD_DATA_ID(n)				((n)<<4)	// The address counter is automatically incremented by 1
#define LCD_DATA_ID_HdecVdec		(0<<4)		// Horizontal: decrement Vertical: decrement
#define LCD_DATA_ID_HincVdec		(1<<4)		// Horizontal: increment Vertical: decrement
#define LCD_DATA_ID_HdecVinc		(2<<4)		// Horizontal: decrement Vertical: increment
#define LCD_DATA_ID_HincVinc		(3<<4)		// Horizontal: increment Vertical: increment
#define LCD_DATA_AM(n)				((n)<<3)	// Set the direction in which the address counter is updated automatically after data are written to the GDDRAM
#define LCD_DATA_AM_H				(0<<3)		// Horizontal
#define LCD_DATA_AM_V				(1<<3)		// Vertical
#define LCD_DATA_LG(n)				((n)<<0)	// Write data to the GDDRAM after comparing the write data written to the GDDRAM by the microcomputer with the values in the compare registers (CPR, CPG, CPB) and performing a logical and arithmetic operation on them.

#define LCD_COMMAND_CompareRegister1				0x04
#define LCD_DATA_CompareRegister1					0b0000000000000000
#define LCD_DATA_CPR(n)				((n)<<10)	// Red
#define LCD_DATA_CPG(n)				((n)<<2)	// Green

#define LCD_COMMAND_CompareRegister2				0x05
#define LCD_DATA_CompareRegister2					0b0000000000000000
#define LCD_DATA_CPB(n)				((n)<<2)	// Blue

#define LCD_COMMAND_DisplayControl					0x07
#define LCD_DATA_DisplayControl						0b0000000000000000
#define LCD_DATA_PT(n)				((n)<<11)	// Normalize the source outputs when non-displayed area of the partial display is driven.
#define LCD_DATA_VLE2				(1<<10)		// a vertical scroll is performed in the 1st screen by taking data VL2 register
#define LCD_DATA_VLE1				(1<<9)		// a vertical scroll is performed in the 1st screen by taking data VL1 register
#define LCD_DATA_SPT				(1<<8)		// 2-division LCD drive is performed.
#define LCD_DATA_GON				(1<<5)		// Gate output enable
#define LCD_DATA_DTE				(1<<4)		// selected gate wire becomes VGH, and non-selected gate wires become VGOFFL
#define LCD_DATA_CM					(1<<3)		// 8-color mode is selected.
#define LCD_DATA_D1					(1<<1)		// Source output enable
#define LCD_DATA_D0					(1<<0)		// Internal display operation enable

#define LCD_COMMAND_FrameCycleControl				0x0b
#define LCD_DATA_FrameCycleControl					0b0000000000000000
#define LCD_DATA_NO(n)				((n)<<14)	// Sets amount of non-overlap of the gate output
#define LCD_DATA_NO_0clockCycle		((0)<<14)	//
#define LCD_DATA_NO_4clockCycle		((1)<<14)	//
#define LCD_DATA_NO_6clockCycle		((2)<<14)	//
#define LCD_DATA_NO_8clockCycle		((3)<<14)	//
#define LCD_DATA_SDT(n)				((n)<<12)	// Set delay amount from the gate output signal falling edge of the source outputs
#define LCD_DATA_SDT_1clockCycle	((0)<<12)	//
#define LCD_DATA_SDT_2clockCycle	((1)<<12)	//
#define LCD_DATA_SDT_3clockCycle	((2)<<12)	//
#define LCD_DATA_SDT_4clockCycle	((3)<<12)	//
#define LCD_DATA_EQ(n)				((n)<<10)	// Sets the equalizing period
#define LCD_DATA_EQ_NoEQ			((0)<<10)	//
#define LCD_DATA_EQ_1clockCycle		((1)<<10)	//
#define LCD_DATA_EQ_2clockCycle		((2)<<10)	//
#define LCD_DATA_EQ_3clockCycle		((3)<<10)	//
#define LCD_DATA_DIV(n)				((n)<<8)	// Set the division ratio of clocks for internal operation
#define LCD_DATA_DIV_2				((0)<<8)	//
#define LCD_DATA_DIV_4				((1)<<8)	//
#define LCD_DATA_DIV_8				((2)<<8)	//
#define LCD_DATA_DIV_16				((3)<<8)	//
#define LCD_DATA_SDIV				(1<<5)		// DIV value will be count
#define LCD_DATA_SRTN				(1<<4)		// RTN value will be count
#define LCD_DATA_RTN(n)				((n)<<0)	// Set the no. of clocks in each line

#define LCD_COMMAND_PowerControl1					0x10
#define LCD_DATA_PowerControl1						0b0000000111000000
#define LCD_DATA_DOT				(1<<15)		// all dcdc clock derivation will use dotclk as clock source Dmode[1:0] = 01 10 11
#define LCD_DATA_DCY(n)				((n)<<12)	// Step-up cycle
#define LCD_DATA_DCY_FlineX8		((0)<<12)	// line frequency
#define LCD_DATA_DCY_FlineX4		((1)<<12)	//
#define LCD_DATA_DCY_FlineX2		((2)<<12)	// [POR]
#define LCD_DATA_DCY_FlineX1		((3)<<12)	//
#define LCD_DATA_DCY_fosc4			((4)<<12)	// internal oscillator frequency
#define LCD_DATA_DCY_fosc8			((5)<<12)	//
#define LCD_DATA_DCY_fosc16			((6)<<12)	//
#define LCD_DATA_DCY_fosc32			((7)<<12)	//
#define LCD_DATA_BTH(n)				((n)<<9)	// VGH output
#define LCD_DATA_BTH_8V				((0)<<9)	//
#define LCD_DATA_BTH_9V				((1)<<9)	//
#define LCD_DATA_BTH_10V			((2)<<9)	//
#define LCD_DATA_BTH_11V			((3)<<9)	//
#define LCD_DATA_BTH_12V			((4)<<9)	//
#define LCD_DATA_BTH_13V			((5)<<9)	//
#define LCD_DATA_BTH_14V			((6)<<9)	//
#define LCD_DATA_BTH_unregulated	((7)<<9)	// [POR]
#define LCD_DATA_DC(n)				((n)<<4)	// Set the step-up cycle of the step-up circuit for VCIX2
#define LCD_DATA_DC_fosc			((0)<<4)	// [POR]
#define LCD_DATA_DC_fosc2			((1)<<4)	//
#define LCD_DATA_DC_fosc4			((2)<<4)	//
#define LCD_DATA_DC_fosc8			((3)<<4)	//
#define LCD_DATA_AP(n)				((n)<<1)	// Op-amp power
#define LCD_DATA_AP_Least			((0)<<1)	//
#define LCD_DATA_AP_Small			((1)<<1)	//
#define LCD_DATA_AP_SmallToMedium	((2)<<1)	// [POR]
#define LCD_DATA_AP_Medium			((3)<<1)	//
#define LCD_DATA_AP_MediumToLarge	((4)<<1)	//
#define LCD_DATA_AP_Large			((5)<<1)	//
#define LCD_DATA_AP_Maximum			((6)<<1)	//
#define LCD_DATA_SLP				(1<<0)		// the driver enters into the sleep mode

#define LCD_COMMAND_PowerControl2					0x11
#define LCD_DATA_PowerControl2						0b0000000000000100
#define LCD_DATA_PU_X3				((0)<<3)	// step up multiplying ratio of VGH from VCI.
#define LCD_DATA_PU_X4				((1)<<3)	//
#define LCD_DATA_PU_X5				((2)<<3)	//
#define LCD_DATA_PU_X6				((3)<<3)	// [POR]

#define LCD_COMMAND_PowerControl3					0x12
#define LCD_DATA_PowerControl3						0b0000011000000000
#define LCD_DATA_SX263B				(1<<8)		// Will not short VCIX2 to VLCD63
#define LCD_DATA_V63SH				(1<<7)		// Works together with VRH define amplitude magnification of VLCD63 from 1.33 to 2.775 times
#define LCD_DATA_VRH(n)				((n)<<0)	// Set amplitude magnification of VLCD63 [POR=10]

#define LCD_COMMAND_PowerControl4					0x13
#define LCD_DATA_PowerControl4						0b0000000000000000
#define LCD_DATA_VCOMG				(1<<13)		// it is possible to set output voltage of VcomL to any level, and the instruction VDV becomes available
#define LCD_DATA_VDV(n)				((n)<<8)	// Set the alternating amplitudes of Vcom at the Vcom alternating drive [POR=16]

#define LCD_COMMAND_HorizontalPorch					0x16
#define LCD_DATA_HorizontalPorch					0b0000000000000000
#define LCD_DATA_XL(n)				((n)<<8)	// Set the number of valid pixel per line [POR=0x83]
#define LCD_DATA_HBP(n)				((n)<<0)	// Set the delay period from falling edge of HSYNC signal to first valid data [POR=0x02]

#define LCD_COMMAND_VerticalPorch					0x17
#define LCD_DATA_VerticalPorch						0b0000000000000000
#define LCD_DATA_VFP(n)				((n)<<8)	// Set the delay period from the last valid line to the falling edge of VSYNC of the next frame [POR=0x04]
#define LCD_DATA_VBP(n)				((n)<<0)	// Set the delay period from falling edge of VSYNC to first valid line [POR=0x02]

#define LCD_COMMAND_PowerControl5					0x1e
#define LCD_DATA_PowerControl5						0b0000000000000000
#define LCD_DATA_nOTP				(1<<7)		// setting of VCM becomes valid and voltage of VcomH can be adjusted regardless VCMR values
#define LCD_DATA_VCM(n)				((n)<<0)	// To set the VcomH voltage if nOTP=1 [POR=0x28]

#define LCD_COMMAND_PowerControl6					0x1f
#define LCD_DATA_PowerControl6						0b0000000000000000
#define LCD_DATA_VCMR(n)			((n)<<0)	// To set the VcomH default voltage if nOTP=0 [POR=0x36]

#define LCD_COMMAND_RAMaddressSet 					0x21
#define LCD_DATA_RAMaddressSet						0b0000000000000000
#define LCD_DATA_X(n)				((n)<<0)	// Make initial settings for the GDDRAM address in the address counter (AC)
#define LCD_DATA_Y(n)				((n)<<8)	//

#define LCD_COMMAND_WriteDataToGRAM					0x22
#define LCD_COMMAND_ReadDataToGRAM					0x22
#define LCD_COMMAND_RAMwriteDataMask1				0x23
#define LCD_COMMAND_RAMwriteDataMask2				0x24
#define LCD_COMMAND_VCOM_OTP1						0x28
#define LCD_COMMAND_VCOM_OTP2						0x29
#define LCD_COMMAND_GammaControl1					0x30
#define LCD_COMMAND_GammaControl2					0x31
#define LCD_COMMAND_GammaControl3					0x32
#define LCD_COMMAND_GammaControl4					0x33
#define LCD_COMMAND_GammaControl5					0x34
#define LCD_COMMAND_GammaControl6					0x35
#define LCD_COMMAND_GammaControl7					0x36
#define LCD_COMMAND_GammaControl8					0x37
#define LCD_COMMAND_GammaControl9					0x38
#define LCD_COMMAND_GammaControl10					0x39
#define LCD_COMMAND_GateScanStartingPosition		0x40
#define LCD_COMMAND_VerticalScrollControl			0x41
#define LCD_COMMAND_FirstDisplayDrivePosition		0x42
#define LCD_COMMAND_SecondDisplayDrivePosition		0x43
#define LCD_COMMAND_HorizontalRAMaddressPosition	0x44
#define LCD_COMMAND_VerticalRAMaddressPosition		0x45
#define LCD_COMMAND_FurtherBias						0x27
#define LCD_COMMAND_OscillatorFrequency				0x2c


#endif /* LCD_SPEC_SSD1283A_H_ */
