//
//  lcd_spec_ST7735S.h	Sitronix Single-Chip TFT Controller/Driver ST7735S command code
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//   Reference:
//      Sitronix Technology Corp.
//      132RGB x 162dot 262K Color with Frame Memory
//      Single-Chip TFT Controller/Driver
//          Datasheet
//              Version     : 1.1
//              Document No.: none
//              Date        : 2011/11
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.07 v0.0  First cording
//

#ifndef LCD_SPEC_ST7735S_H_
#define LCD_SPEC_ST7735S_H_

//
// System Function Command
//
#define LCD_COMMAND_NOP			0x00	// No operation

#define LCD_COMMAND_SWRESET		0x01	// Software reset

#define LCD_COMMAND_SLPIN		0x10	// Sleep in & booster off

#define LCD_COMMAND_SLPOUT		0x11	// Sleep out & booster on

#define LCD_COMMAND_PTLON		0x12	// Partial mode on

#define LCD_COMMAND_NORON		0x13	// Partial off (normal)

#define LCD_COMMAND_INVOFF		0x20	// Display inversion off (normal)

#define LCD_COMMAND_INVON		0x21	// Display inversion on

#define LCD_COMMAND_GAMSET		0x26	// Gamma curve select (1byte)
#define   LCD_DATA_GC0				0x01	//
#define   LCD_DATA_GC1				0x02	//
#define   LCD_DATA_GC2				0x04	//
#define   LCD_DATA_GC3				0x08	//

#define LCD_COMMAND_DISPOFF		0x28	// Display off

#define LCD_COMMAND_DISPON		0x29	// Display on

#define LCD_COMMAND_CASET		0x2A	// Column address set (4byte)
#define   LCD_DATA_X(s,e)			((s)>>8),((s)&0xff),((e)>>8),((e)&0xff)

#define LCD_COMMAND_RASET		0x2B	// Row address set (4byte)
#define   LCD_DATA_Y(s,e)			((s)>>8),((s)&0xff),((e)>>8),((e)&0xff)

#define LCD_COMMAND_RAMWR		0x2C	// Memory write (n word)

#define LCD_COMMAND_RGBSET		0x2D	// LUT for indexed color (128byte)

#define LCD_COMMAND_PTLAR		0x30	// Partial start/end address set (4byte)
#define   LCD_DATA_PL(s,e)			((s)>>8),((s)&0xff),((e)>>8),((e)&0xff)

#define LCD_COMMAND_SCRLAR		0x33	// Scroll area set (6byte)
#define   LCD_DATA_TFA(n)			((n)>>8),((n)&0xff)
#define   LCD_DATA_VSA(n)			((n)>>8),((n)&0xff)
#define   LCD_DATA_BFA(n)			((n)>>8),((n)&0xff)

#define LCD_COMMAND_TEOFF		0x34	// Tearing effect line off

#define LCD_COMMAND_TEON		0x35	// Tearing effect mode set & on (1byte)
#define  LCD_DATA_TEM(n)			(n)

#define LCD_COMMAND_MADCTL		0x36	// Memory data access control (1byte)
#define   LCD_DATA_MY				(1<<7)	// Row Address Order
#define   LCD_DATA_MX				(1<<6)	// Column Address Order
#define   LCD_DATA_MV				(1<<5)	// Row/Column Exchange
#define   LCD_DATA_ML				(0<<4)	// Vertical Refresh Order
#define   LCD_DATA_ML_Top_Bottom	(0<<4)	//
#define   LCD_DATA_ML_Bottom_Top	(1<<4)	//
#define   LCD_DATA_RGB				(0<<3)	// RGB-BGR ORDER
#define   LCD_DATA_BGR				(1<<3)	// RGB-BGR ORDER
#define   LCD_DATA_MH				(1<<2)	// Horizontal Refresh Order
#define   LCD_DATA_MH_Left_Right	(0<<2)	//
#define   LCD_DATA_MH_Right_Left	(1<<2)	//

#define LCD_COMMAND_VSCSAD		0x37	// Scroll start address of RAM (2byte)
#define   LCD_DATA_SSA(n)			((n)>>8),((n)&0xff)

#define LCD_COMMAND_IDMOFF		0x38	// Idle mode off

#define LCD_COMMAND_IDMON		0x39	// Idle mode on

#define LCD_COMMAND_COLMOD		0x3A	// Interface pixel format (1byte)
#define   LCD_DATA_IFPF(n)			(n)	//
#define   LCD_DATA_IFPF_12BIT		3	// 4k
#define   LCD_DATA_IFPF_16BIT		5	// 65k
#define   LCD_DATA_IFPF_18BIT		6	// 262k

//
// Panel Function Command
//
#define LCD_COMMAND_FRMCTR1		0xB1	// Frame Rate Control 1 (3byte)
#define   LCD_DATA_RTNA(n)			(n)	// 0..15
#define   LCD_DATA_FPA(n)			(n)	// 0..63
#define   LCD_DATA_BPA(n)			(n)	// 0..63
#define LCD_COMMAND_FRMCTR2		0xB2	// Frame Rate Control 2 (3byte)
#define LCD_COMMAND_FRMCTR3		0xB3	// Frame Rate Control 3 (6byte)
#define LCD_COMMAND_INVCTR		0xB4	// Display Inversion Control (1byte)

#define LCD_COMMAND_PWCTR1		0xC0	// PowerControlSetting
#define LCD_COMMAND_PWCTR2		0xC1	// PowerControlSetting
#define LCD_COMMAND_PWCTR3		0xC2	// PowerControlSetting
#define LCD_COMMAND_PWCTR4		0xC3	// PowerControlSetting
#define LCD_COMMAND_PWCTR5		0xC4	// PowerControlSetting
#define LCD_COMMAND_VMCTR1		0xC5	// VCOMControl1
#define LCD_COMMAND_VMOFCTR		0xC7	// Set VCOM Offset control
#define LCD_COMMAND_WRID2		0xD1	// Set LCM Version Code
#define LCD_COMMAND_WRID3		0xD2	// CustomerProject Code
#define LCD_COMMAND_NVCTR1		0xD9	// NVM Control Status
#define LCD_COMMAND_NVCTR2		0xDE	// NVM Read Command
#define LCD_COMMAND_NVCTR3		0xDF	// NVM Write Command Action Code
#define LCD_COMMAND_GAMCTRP1	0xE0	// Gamma Adjustment
#define LCD_COMMAND_GAMCTRN1	0xE1	// Gamma Adjustment
#define LCD_COMMAND_GCV			0xFC	// Gate Pump Clock Frequency Variable


#endif /* LCD_SPEC_ST7735S_H_ */
