//
//  lcd_touch_driver.h	Part of Graphic LCD module I/O library, Touch screen driver
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	  Description
//		I/O control for Graphic LCD module.
//      Touch screen data transfer subroutines.
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.19 v0.0  First cording
//

#ifndef LCD_TOUCH_DRIVER_H_
#define LCD_TOUCH_DRIVER_H_

//
//	Configuration
//
#define LCD_TOUCH_DRIVER_POINT_MAX	4095

#if defined(LCD_MODULE_CP2401T) || defined(LCD_MODULE_CP2401T_C)
#define LCD_DRIVER_4WIRE_TOUCH
#define AA_Y1	450		// Offset
#define AA_Y2	3700	// Max
#define AA_X1	750		// Offset
#define AA_X2	3100	// Max
#define AA_Z1	700		// Offset
#define AA_Z2	3000	// Max
#define AA_ZY	-30		// [%] Y axis relation rate
#define AA_ZX	30		// [%] X axis relation rate

#elif defined(LCD_MODULE_S95417) || defined(LCD_MODULE_S95417AAA)
#define LCD_DRIVER_4WIRE_TOUCH
#define AA_Y1	450		// Offset
#define AA_Y2	3700	// Max
#define AA_X1	750		// Offset
#define AA_X2	3100	// Max
#define AA_Z1	700		// Offset
#define AA_Z2	3000	// Max
#define AA_ZY	-30		// [%] X axis relation rate
#define AA_ZX	30		// [%] Z axis relation rate

#elif defined(LCD_MODULE_S95160)
#define LCD_DRIVER_4WIRE_TOUCH
#define AA_Y1	270		// Offset
#define AA_Y2	3700	// Max
#define AA_X1	410		// Offset
#define AA_X2	3700	// Max
#define AA_Z1	700		// Offset
#define AA_Z2	3000	// Max
#define AA_ZY	-40		// [%] Y axis relation rate
#define AA_ZX	30		// [%] X axis relation rate

#elif defined(LCD_MODULE_QJ240083S0)
#define LCD_DRIVER_4WIRE_TOUCH
#define AA_Y1	270		// Offset
#define AA_Y2	3700	// Max
#define AA_X1	460		// Offset
#define AA_X2	3550	// Max
#define AA_Z1	700		// Offset
#define AA_Z2	3000	// Max
#define AA_ZY	-40		// [%] Y axis relation rate
#define AA_ZX	30		// [%] X axis relation rate

#elif defined(LCD_MODULE_MSP2807)
#define LCD_DRIVER_XPT2046
#define AA_Y1	200		// Offset
#define AA_Y2	3750	// Max
#define AA_X1	250		// Offset
#define AA_X2	3850	// Max
#define AA_Z1	100		// Offset
#define AA_Z2	3500	// Max
#define AA_ZY	-30		// [%] Y axis relation rate
#define AA_ZX	-20		// [%] X axis relation rate

#else
	// Touch offline
#endif

//
// API
//
extern int	lcd_touchDriverStart(void);				// return < 0 : Error
extern int	lcd_touchDriverGet(short point[3]);		// return < 0 : Error, 0:point is valid, 1:busy
extern int	lcd_touchDriverStop(void);				// return < 0 : Error

#endif	// LCD_TOUCH_DRIVER_H_
