//
//  lcd_touch_port.h	Part of Graphic LCD module I/O library, Touch screen port I/O
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	  Description
//		I/O control for Graphic LCD module.
//      Touch screen data transfer subroutines.
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.19 v0.0  First cording
//

#ifndef LCD_TOUCH_PORT_H_
#define LCD_TOUCH_PORT_H_

#include <stdint.h>
#include <stddef.h>

//
// Argument
//
enum lcd_touchPortId {
	PORT_YU = 0,
	PORT_YD,
	PORT_XL,
	PORT_XR,
};
enum lcd_touchPortMode {
	PORT_OUTPUT = 0,
	PORT_INPUT,
	PORT_ANALOG,
};

//
// Prototype
//
extern void		lcd_touchPortStart(void);
extern void		lcd_touchPortStop(void);

extern void		lcd_touchPortSet(enum lcd_touchPortId port, enum lcd_touchPortMode mode, int onOff);
extern int		lcd_touchPortGetAnalog(enum lcd_touchPortId port);

extern size_t	lcd_touchPortSend(size_t len, unsigned char *data);
extern size_t	lcd_touchPortRecv(size_t len, unsigned char *data);
extern void		lcd_touchPortCS(int	flag);

extern uint32_t	lcd_touchPortGetTick(void);

#endif	// LCD_TOUCH_PORT_H_
